
package com.soaresvictor;

import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.WritableMap;
import com.model.ControlOrder;
import com.model.Coo;
import com.model.NumeroSerieAgente;
import com.model.NFe;
import com.model.Response;
import com.tcpclient.TcpClient;
import com.templates.Templates;

import java.util.List;
import java.util.Random;

public class RNSocketTcpClientModule extends ReactContextBaseJavaModule {

    private static final String PKG = "com.socket.RN";
    private Random rng;
    private Templates templates;
    private Integer MAX_INT = 1000000000;

    public RNSocketTcpClientModule(ReactApplicationContext reactContext) {
        super(reactContext);

        rng = new Random();
        templates = new Templates();
    }

    @Override
    public String getName() {
        return "RNSocketTcpClient";
    }

    @ReactMethod
    public void pedeCOO(Integer filial, Integer pdv, String host, int port, Promise p) {
        try {
            String agentKey = templates.getAgentKey(filial, pdv, rng.nextInt(MAX_INT));
            String header = templates.getHeader(agentKey, "ped_coo-number");
            String data = header.concat(templates.getCooMsg(agentKey));

            new Thread(new TcpClient(host, port, data, null, this, p)).start();
        } catch (Exception e) {
            Log.e(PKG, "Unexpected error:", e);
        }
    }

    @ReactMethod
    public void pedeNumeroSerieAgente(Integer filial, Integer pdv, String host, int port, Promise p) {
        try {
            String agentKey = templates.getAgentKey(filial, pdv, rng.nextInt(MAX_INT));
            String header = templates.getHeader(agentKey, "ped_status-sat");
            String data = header.concat(templates.getCooMsg(agentKey));

            new Thread(new TcpClient(host, port, data, null, this, p)).start();
        } catch (Exception e) {
            Log.e(PKG, "Unexpected error:", e);
        }
    }

    @ReactMethod
    public void enviaNFE(Integer filial, Integer pdv, String xml, String host, int port, Promise p) {
        try {
            String agentKey = templates.getAgentKey(filial, pdv, rng.nextInt(MAX_INT));
            String header = templates.getHeader(agentKey, "ped_env");
            String data = header.concat(xml);

            new Thread(new TcpClient(host, port, data, null, this, p)).start();
        } catch (Exception e) {
            Log.e(PKG, "Unexpected error:", e);
        }
    }

    @ReactMethod
    public void cancelaNFE(Integer filial, Integer pdv, String xml, String host, int port, Promise p) {
        try {
            String agentKey = templates.getAgentKey(filial, pdv, rng.nextInt(MAX_INT));
            String header = templates.getHeader(agentKey, "ped_can-inu");
            String data = header.concat(xml);

            new Thread(new TcpClient(host, port, data, null, this, p)).start();
        } catch (Exception e) {
            Log.e(PKG, "Unexpected error:", e);
        }
    }

    @ReactMethod
    public void impressaoAdicional(Integer filial, Integer pdv, String xml, String host, int port, Promise p) {
        try {
            String agentKey = templates.getAgentKey(filial, pdv, rng.nextInt(MAX_INT));
            String header = templates.getHeader(agentKey, "ped_print-adic");
            String data = header.concat(xml);

            new Thread(new TcpClient(host, port, data, null, this, p)).start();
        } catch (Exception e) {
            Log.e(PKG, "Unexpected error:", e);
        }
    }

    public void resolve(List data, Promise p) {
        WritableMap ret = Arguments.createMap();
        boolean success = false;
        for (Object o : data) {
            if (o.getClass().equals(ControlOrder.class)) {
                ret = ((ControlOrder) o).toWritableArray(ret);
            } else if (o.getClass().equals(Coo.class)) {
                ret = ((Coo) o).toWritableArray(ret);
            } else if (o.getClass().equals(NumeroSerieAgente.class)) {
              ret = ((NumeroSerieAgente) o).toWritableArray(ret);
            } else if (o.getClass().equals(NFe.class)) {
                ret = ((NFe) o).toWritableArray(ret);
            } else if (o.getClass().equals(Response.class)) {
                ret = ((Response) o).toWritableArray(ret);

                success = ((Response) o).code == 100;
            }
        }

        if (success) {
            p.resolve(ret);
        } else {
            p.reject(String.valueOf(ret.getInt("code")), ret.getString("description"));
        }
    }

    public void reject(Promise p, Throwable e) {
        p.reject(e);
    }
}
