package com.tcpclient;

import android.support.annotation.Nullable;
import android.util.Log;

import com.facebook.react.bridge.Promise;
import com.soaresvictor.RNSocketTcpClientModule;
import com.xmlparser.NDDParser;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.util.List;


public class TcpClient implements Runnable {

    private final String host;
    private final int port;
    private final String stringData;
    private final byte[] byteData;
    private static final String PKG = "br.com.rchlo.socket";
    private RNSocketTcpClientModule context;
    private Socket socket;
    private Promise promise;

    public TcpClient(String host, int port, @Nullable String stringData, @Nullable byte[] byteData, RNSocketTcpClientModule ctx, Promise p) throws Exception {
        this.host = host;
        this.port = port;
        this.stringData = stringData;
        this.byteData = byteData;
        this.context = ctx;
        this.promise = p;

        if(stringData == null && byteData == null){
            throw new Exception("Nenhum argunmento de dado foi informado");
        }
    }

    @Override
    public void run() {
        if (this.stringData != null) {
            try {
                this.send(this.stringData);
            } catch (Exception ex) {
                Log.e(PKG, ex.getMessage(), ex);
            }
        } else {
            try {
                this.send(this.byteData);
            } catch (Exception ex) {
                Log.e(PKG, ex.getMessage(), ex);
            }
        }
    }

    private void send(String data) throws Exception {
        try {
            socket = new Socket(this.host, this.port);
            //1 minute
            socket.setSoTimeout(60000);
            Log.d(PKG, socket.getSoTimeout() + " secs timeout");

            DataOutputStream dOut = new DataOutputStream(socket.getOutputStream());
            OutputStreamWriter osw = new OutputStreamWriter(dOut);
            BufferedWriter bo = new BufferedWriter(osw);
            Log.d("SocketClient", "IN: " + data);
            bo.write(new String(data.getBytes(), "UTF-8"));
            bo.flush();
            socket.shutdownOutput();

            InputStream is = socket.getInputStream();

            BufferedReader br = new BufferedReader(new InputStreamReader(is));
            // Skip first line, it contains smthing like #RCHLO....
            // that is an invalid XML
            String ln = br.readLine();
            StringBuilder sb = new StringBuilder();
            while(ln != null) {
                ln = br.readLine();
                Log.d(PKG, "FOUND: " + ln);

                if(ln != null)
                    sb.append(ln);
            }

            NDDParser ans = new NDDParser();
            List ret = ans.parse(new ByteArrayInputStream(sb.toString().getBytes()));
            context.resolve(ret, promise);
            
            dOut.close();

        } catch (IOException e) {
            Log.e(PKG, e.getMessage(), e);
            context.reject(promise, e);
        } finally {
            try {
                Log.w(PKG, "FINALIZING");
                //socket.close();

            } catch (Exception e) {
                Log.e(PKG, e.getMessage(), e);
            }
        }
    }

    private void send(byte[] data) {
        try {
            socket = new Socket(this.host, this.port);

            DataOutputStream dOut = new DataOutputStream(socket.getOutputStream());

            dOut.write(data);
            dOut.flush();
        } catch (IOException e) {
            System.out.println(e.getMessage());
        } finally {
            try {
                socket.close();
            } catch (IOException e) {
                Log.e(PKG, e.getMessage(), e);
            }
        }
    }
}
